#!/usr/bin/env python2
__author__ = 'jon'


import subprocess
import os


versioning = ["Kernel version",
              "Distro",
              "Software Version",
              "Firmware Version"]

station = ["Station Name",
           "Channel Groups",
           "Data-Flow Mode",
           "FDSN Network Name",
           "FDSN Network Description",
           "FDSN Station Name",
           "FDSN Station Description",
           "FDSN Location Name",
           "FDSN Location Description",
           "Seismometer Sample Rate",
           "Acceleromter Sample Rate",
           "GPS",
           "GPS Coordinates",
           "Event Triggers (ON|OFF)"]

seedlink = ["Port Number"]

comms = ["DHCP",
         "IP Address",
         "IP Discovery"]

bbbInfo = ["BBB revision",
           "MAC",
           "System time",
           "Time Since Last Restart",
           "Disk usage"]


maxAlignment = 30


def prettyprint(key, value):
    display_string = key + " : "
    if len(key) > maxAlignment:
        spaces = 0
    else:
        diff = maxAlignment - len(display_string)
        spaces = diff

    with_space = " " * spaces + display_string + value + "\n"
    return with_space


def writeInfo(outreport, title, list):
    outreport += "\n\n%s\n" % title
    outreport += "-" * len(title) + "\n\n"
    for key in list:
        if key not in report.keys():
            continue
        outreport += prettyprint(key, report[key].strip())

    return outreport



def generateReport(report):

    outreport = "MySixaola report\n"
    outreport += "=" * len("MySixaola report")
    outreport = writeInfo(outreport, "Versioning", versioning)
    outreport = writeInfo(outreport, "Computer info", bbbInfo)
    outreport = writeInfo(outreport, "Station info", station)
    if "Port Number" in report.keys():
        outreport = writeInfo(outreport, "%s info" % report["Data-Flow Mode"], seedlink)
    outreport = writeInfo(outreport, "Communications", comms)

    return outreport



def parseBlueconf(report= {}):
    info = subprocess.check_output("/opt/bin/blueconf --info".split())

    for line in info.split("\n"):
        if ':' in line:
            lineSplit = line.split(":")
            #if len(lineSplit) < 2:
            #    continue
            report[lineSplit[0].strip()] = lineSplit[1].strip()

    return report


def diskUsage():
    p = os.popen('''df -h | awk '{ if ($6 == "/") print $5 " used, " $4 " available" }' ''')
    l = p.readline().strip()
    return l


def main():


    report = parseBlueconf()

    report["Kernel version"] = subprocess.check_output("cat /proc/version".split())

    if "Debian" in report["Kernel version"]:
        report["Distro"] = "Debian"
    elif "Arch" in report["version"]:
        report["Distro"] = "Debian"

    report["BBB revision"] = subprocess.check_output("sudo cat /sys/devices/bone_capemgr.9/baseboard/revision".split()).split("0")[-1]

    ifconfig = subprocess.check_output("sudo ifconfig".split())

    if report["Distro"] == "Debian":
        keyword = "HWaddr"
    else:
        keyword  = "ether"

    report["MAC"] = ifconfig.split(keyword)[1].split()[0]

    if report["Distro"] == "Debian":
        keyword = "addr:"
    else:
        keyword  = "inet "

    ## Static or dynamic?
    if report["Distro"] == "Debian":
        iface = "/etc/network/interfaces"
    else:
        iface  = "/etc/netctl/eth0"

    with open(iface, 'r') as ifaceFile:
        ifaceInfo = ifaceFile.readlines()


    report["DHCP"] = "dynamic IP (DHCP)"

    for line in ifaceInfo:
        if "static" in line:
            report["DHCP"] = "static IP"

    # GPS coordinates
    if os.path.isfile("/opt/settings/sc-xml/GPSCoords.txt"):
        gpsFile = open("/opt/settings/sc-xml/GPSCoords.txt", 'r')
        lines = gpsFile.read().split()
        if len(lines) >= 3:
            lat, lon, alt = lines[0:3]
            report["GPS Coordinates"] = 'lat = %s, lon = %s, alt = %s' % (lat, lon, alt)
        else:
            report["GPS Coordinates"] = 'Not available'
        gpsFile.close()

    report["Disk usage"] = diskUsage()

    return report



if __name__ == "__main__":

    report = main()

    doc = generateReport(report)

    print doc
