from Config import globals
import re

class GenericComponent:

    def __init__(self, firstTime):
        self.changedSettings = []
        self.oldSettings = []
        self.firstTime = firstTime
        self.required = True
        self.OnOffMatch = "^(ON|OFF)$"
        self.numberMatch = "[+-]?((\d+(\.\d*)?)|\.\d+)([eE][+-]?[0-9]+)?"
        self.cidrMatch = "^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\/(\d|[1-2]\d|3[0-2]))$"
        self.ipMatch = "^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$"

    def validate(self, validation, input_val, lengthLimit = 0, required = False):
        """
        Take a validation requirement and validate
        it accordingly.
        """

        if not input_val and required:
            return False

        if not validation:
            return True

        if lengthLimit > 0 and len(input_val) > lengthLimit:
            return False

        prog = re.compile(validation)

        if prog.match(input_val):
            return True

        # print input_val
        # print " does not match"
        # print validation
        return False

    def validateAndRead(self, validation, input_val, display_string, lengthLimit = 0, required = False):
        """
        Check if passed input is valid. If
        it's not, then ask for new input
        until it is.
        """
        # read until we get a correct value
        while True:
            if self.validate(validation, input_val, lengthLimit, required):
                return input_val

            if '(\\/(\\d|[1-2]\\d|3[0-2]))$' in validation:
                print "Please re-enter IP in format 10.18.100.234/24, note the last '/24'"
            elif required:
                print "Invalid entry. Please re-enter"
            elif not input_val:
                return input_val
            else:
                print "Invalid entry. Please re-enter"

            input_val = raw_input(display_string)


    # Displays and reads values based on alignment
    # Dictionary is the dictionary
    # value is the value at the dictionary we want to read
    # by default it will display it
    def displayAndRead(self, label, dictionary, value, requires = [1,2,3,4,5,6], validation = "", displayOnly = False, lengthLimit = 0, required = False, useDefault = False, transform = None):


        if displayOnly is True:
            self.displayOnly(label, dictionary[value], requires)
            return

        if globals.EXEC_MODE not in requires:
            return


        display_string = label + " [" + dictionary[value] + "] : "

        if len(display_string) > self.maxAlignment:
            spaces = 0
        else:
            diff = self.maxAlignment - len(display_string)
            spaces = diff

        with_space = " " * spaces + display_string

        new_val = raw_input(with_space)



        new_val = self.validateAndRead(validation, new_val, with_space, lengthLimit, required)

        if transform is not None and new_val is not '':
            new_val = transform(new_val)

        if new_val == '':
            return
        elif self.firstTime is not True:
            old_string = " [" + dictionary[value] + "]"
            new_string = " [" + new_val + "]"

            strings = self.correctStrings(old_string, new_string)
            self.oldSettings.append(strings[0] + " : " + label)
            dictionary[value] = new_val
            self.changedSettings.append(strings[1] + " : " + label)
        else:

            dictionary[value] = new_val

    def correctStrings(self, old_string, new_string):
        """
        Make two strings the same length
        """
        diff = self.getDiff(old_string, new_string)

        if len(old_string) > len(new_string):
            new_string = new_string + " " * diff
        elif len(new_string) > len(old_string):
            old_string = old_string + " " * diff

        strings = [old_string, new_string]
        return strings

    def setRequired(self, value):
        if not value:
            return True
        else:
            return False




    def getDiff(self, A, B):
        """
        Get the difference between two strings
        """

        A = len(A)
        B = len(B)

        if A > B:
            return A - B
        elif B > A:
            return B - B
        else:
            return 0


    # Displays the old values
    # and the new values and
    # prompts the user to save or not
    # saving will write out the changes
    def displayChanges(self, changed = False):
        if self.firstTime is not True:

            if changed:
                return False
            elif not self.changedSettings:
                print "\nNo changes made"
                return False
            elif self.changedSettings and not self.oldSettings:
                for val in self.changedSettings:
                    print "\n" + val
                return False
            else:
                print "\nChanged settings are:\n"
                for old,new in zip(self.oldSettings, self.changedSettings):
                    print "Previous : " + old
                    print "New      : " + new
                    print "\n"
                # empty both lists
                self.oldSettings = []
                self.changedSettings = []
                return True


    # Displays a value
    # Some values are read only
    def displayOnly(self, label, value, requires = [1,2,3,4,5,6]):
        """
        Only display the value in value with the label
        Requires -> an optional parameter to determine if
        the item should be shown in the current exec mode
        """
        if globals.EXEC_MODE not in requires:
            return

        display_string = label + " : "
        if len(label) > self.maxAlignment:
            spaces = 0
        else:
            diff = self.maxAlignment - len(display_string)
            spaces = diff

        with_space = " " * spaces + display_string + value

        print with_space

    # Determines if the next items should be
    # displayed and displays an error
    # message if they shouldn't
    def determineDisplay(self, requires):
        if globals.EXEC_MODE not in requires:
            return False
        else:
            return True
