from Config import globals
from Util.cli import getSaveChoice
from Component import GenericComponent

class DataSettings(GenericComponent):
    def display(self, displayOnly = False):
        self.justDisplay = displayOnly


        seedlink = globals.CONFIG_SETTINGS.find('SEEDLINK')
        self.displaySeedlink(seedlink)

        caps = globals.CONFIG_SETTINGS.find("CAPS")
        self.displayCaps(caps)

        trigger = globals.CONFIG_SETTINGS.find('TRIGGER')
        self.displayTrigger(trigger)

        slarchive = globals.CONFIG_SETTINGS.find('SLARCHIVE')
        self.displaySlArchive(slarchive)
 
        return self.save(seedlink, trigger, slarchive, caps)
        
    def __init__(self, firstTime):
        self.maxAlignment = 36
        self.trueTable = {
            "ON": True,
            "OFF": False,
            "": False
        }
        GenericComponent.__init__(self, firstTime)
    
    def displaySeedlink(self, seedlink):
        if(self.determineDisplay([1, 5])):
            print "\n\n**** Seedlink ****\n"
            self.displayAndRead("Port Number", seedlink.SCPORT, 'VALUE', [1,5], self.numberMatch, self.justDisplay)

    def displayCaps(self, caps):
        if (self.determineDisplay([6])):
            print "\n\n**** CAPS ****\n"
            self.displayAndRead("Port Number", caps.PORT, "VALUE", [6], self.numberMatch, self.justDisplay)

    def displayTrigger(self, trigger):
        if(self.determineDisplay([1,2,3,4,5])):
            print "\n\n**** Event Trigger ****\n"
            EVENTTRIG = self.displayAndReadEvt(trigger.ONOFF)
            if EVENTTRIG is True:
                required = self.setRequired(trigger.LTATIME['VALUE'])
                self.displayAndRead("Long-Term Average Time (LTA)", trigger.LTATIME, 'VALUE', [1,2,3,4,5], self.numberMatch, self.justDisplay, 100, required)

                required = self.setRequired(trigger.STATIME['VALUE'])
                self.displayAndRead("Short-Term Average Time (STA)", trigger.STATIME, 'VALUE', [1,2,3,4,5], self.numberMatch, self.justDisplay, 100, required)

                required = self.setRequired(trigger.RATIO['VALUE'])
                self.displayAndRead("Ratio", trigger.RATIO, 'VALUE', [1,2,3,4,5], self.numberMatch, self.justDisplay, 100, required)

                required = self.setRequired(trigger.QUIET['VALUE'])
                self.displayAndRead("Quiet", trigger.QUIET, 'VALUE', [1,2,3,4,5], self.numberMatch, self.justDisplay, 100, required)

                required = self.setRequired(trigger.TRIGDUR['VALUE'])
                self.displayAndRead("Trace Length (seconds)", trigger.TRIGDUR, 'VALUE', [1,2,3,4,5], self.numberMatch, self.justDisplay, 100, required)
                    
                required = self.setRequired(trigger.PREEVTTIME['VALUE'])
                self.displayAndRead("Pre-Event Length (seconds)", trigger.PREEVTTIME, 'VALUE', [1,2,3,4,5], self.numberMatch, self.justDisplay, 100, required)

    def displaySlArchive(self, slarchive):
        if(self.determineDisplay([3])):
            print "\n\n**** Continuous Data Archival ****\n"
            self.displayAndRead("Days to Archive ", slarchive.ARCHKEEP, 'VALUE', [3], self.numberMatch, self.justDisplay)

        if (self.determineDisplay([5])):
            print "\n\n**** Continuous Data Archival ****\n"
            print "Days to Archive: Fixed value 3 days.\n"


    def displayAndReadEvt(self, trigger):
        """
        Display the Event Trigger settings, abiding by necessary rules
        """

        if globals.EXEC_MODE == 4:
            # special case that event_logger mode requires all
            # triggers to be on and there should be no option
            trigger['VALUE'] = "ON" # trigger must be on
            return True # make sure that the other settings get showed
            



        label = "Event Triggers (ON|OFF)"

        if self.justDisplay is True:
            self.displayOnly(label, trigger['VALUE'])
            return


        display_string = label + " [" + trigger['VALUE'] + "] : "
        
        if len(display_string) > self.maxAlignment:
            spaces = 0
        else:
            diff = self.maxAlignment - len(display_string)
            spaces = diff

        with_space = " " * spaces + display_string
        


        new_val = raw_input(with_space).upper().replace(" ", "")

           
            
        new_val = self.validateAndRead(self.OnOffMatch, new_val, with_space)

        ret = self.trueTable[new_val.upper()]
        
        if new_val == '':
            return self.trueTable[trigger['VALUE']]
        elif self.firstTime is not True:
            old_string = "[" + trigger['VALUE'] + "]"
            new_string =  "[" + new_val + "]"

            strings = self.correctStrings(old_string, new_string)

            self.oldSettings.append(strings[0] + " : " + label)
            self.changedSettings.append(strings[1] + " : " + label)
            trigger['VALUE'] = new_val
            return ret
        else:
            trigger['VALUE'] = new_val
            return ret


    def save(self, seedlink, trigger, slarchive, caps):
        # If the use is in first time mode
        # just save the changes and continue
        if self.firstTime is False:
            changesMade = self.displayChanges()
            if not changesMade:
                choice = 'n'
                return choice

        
            choice = getSaveChoice()


            if choice == 's':
                globals.CONFIG_SETTINGS.find('SEEDLINK').replaceWith(seedlink)
                globals.CONFIG_SETTINGS.find('TRIGGER').replaceWith(trigger)
                globals.CONFIG_SETTINGS.find('SLARCHIVE').replaceWith(slarchive)
                globals.CONFIG_SETTINGS.find('CAPS').replaceWith(caps)
        
            return choice
        else:
            globals.CONFIG_SETTINGS.find('SEEDLINK').replaceWith(seedlink)
            globals.CONFIG_SETTINGS.find('TRIGGER').replaceWith(trigger)
            globals.CONFIG_SETTINGS.find('SLARCHIVE').replaceWith(slarchive)
            globals.CONFIG_SETTINGS.find('CAPS').replaceWith(caps)

 
