from Config import globals
from Util.cli import getSaveChoice
from Util import util
from Component import GenericComponent

class InstrumentationSettings(GenericComponent):
    def display(self, displayOnly = False):
        
        self.justDisplay = displayOnly

        # Label [Current Value]:
        glob = globals.CONFIG_SETTINGS.find('GLOBAL')
        self.displayGlobals(glob)

        return self.save(glob)

    def __init__(self, firstTime):
        self.maxAlignment = 39
        self.nameMatch = "[A-Z][A-Z0-9]"
        self.stationMatch = "^[A-Z][A-Z0-9]+$"
        self.locationMatch = "[ -~][ -~]"
        self.velMatch = "^(100|50|OFF)$"
        self.accMatch = "^(200|100|50|OFF)$"
        GenericComponent.__init__(self, firstTime)


    def displayGlobals(self, glob):
        print "\n**** Instrument Settings ****\n"
        self.displayAndReadDataFlowMode("Data-Flow Mode", glob.EXECUTIONMODE, 'VALUE')

        required = self.setRequired(glob.NTW['NAME'])

        upperCase = lambda x: x.upper()
        def off(x):
            if x == "OFF":
                return "0" 
            else:
                return x

        self.displayAndRead("FDSN Network NAME", glob.NTW, 'NAME', [1,3,4,5,6], self.nameMatch, self.justDisplay, 2, required, False, upperCase)
        self.displayAndRead("FDSN Network Description", glob.NTW, 'DESC', [1,3,4,5,6], "", self.justDisplay)

        required = self.setRequired(glob.STN['NAME'])
        self.displayAndRead("FDSN Station NAME", glob.STN, 'NAME', [1,3,4,5,6], self.stationMatch, self.justDisplay, 5, required, False, upperCase)

        self.displayAndRead("FDSN Station Description", glob.STN, 'DESC', [1,3,4,5,6], "", self.justDisplay)
        self.displayAndRead("FDSN Location NAME", glob.LOC, 'NAME', [1,3,4,5,6], self.locationMatch, self.justDisplay, 2, False, False, upperCase)
        self.displayAndRead("FDSN Location Description", glob.LOC, 'DESC', [1,3,4,5,6], "", self.justDisplay)

        self.displayAndRead("Logging (ON|OFF)", glob.LOGGING, 'VALUE', [1,2,3,4,5,6], self.OnOffMatch,self.justDisplay)
        self.displayAndRead("VEL Sample Rate (100|50|OFF)", glob.SPSV, 'VALUE', [1,2,3,4,5,6], self.velMatch, self.justDisplay, transform = off)
        self.displayAndRead("ACC Sample Rate (200|100|50|OFF)", glob.SPSA, 'VALUE',[1,2,3,4,5,6], self.accMatch, self.justDisplay, 3, False, False,  off)


    def save(self, glob):
        # If the use is in first time mode
        # just save the changes and continue
        if self.firstTime is False:
            changesMade = self.displayChanges()

            if not changesMade:
                choice = 'n'
                return choice


            choice = getSaveChoice()


            if choice == 's':
                globals.CONFIG_SETTINGS.find('GLOBAL').replaceWith(glob)
                #globals.CONFIG_SETTINGS.find('TIMING').replaceWith(timing)

            return choice

        else:
            globals.CONFIG_SETTINGS.find('GLOBAL').replaceWith(glob)
            #globals.CONFIG_SETTINGS.find('TIMING').replaceWith(timing)

            
    # Identical to displayAndRead
    # execpt it only handle the GLOBAL
    # so we can chagne that value
    def displayAndReadDataFlowMode(self, label, dictionary, value):

        table = {
            'a': 'SEEDLINK',
            'b': 'DARIEN',
            'c': 'DATA_LOGGER',
            'd': 'EVENT_LOGGER',
            'e': "SEEDLINK_LOCAL_BACKUP",
            'f': "CAPS"
        }
        

        display_string = label + " : "
        
        CHOICES_STRING = """
Choices:
   A) SeedLink Streamer
   B) Darien Client
   C) Data Logger
   D) Event Logger
   E) SeedLink Streamer (With Local Backup)
   F) CAPS"""


        spaces = 2
        with_space = " " * spaces + display_string + util.prettyDFMode(dictionary[value])

        if self.justDisplay is True:
            self.displayOnly(label, util.prettyDFMode(dictionary[value]))
            return

        if self.firstTime is not True or dictionary[value] != '':
            print with_space
        else:
            print "Please select a Data-Flow Mode"

        print CHOICES_STRING
        new_val = raw_input("> ").lower().replace(" ", "")

        if new_val == '':
            return

        # validate the input - if it's not valid
        # get the user to keep trying until it is
        while True:
            if new_val not in table:
                # get some new input
                print "Option not valid."
                print CHOICES_STRING
                new_val = raw_input("> ").lower().replace(" ", "")
                continue
            else:
                break

        if self.firstTime is not True:
            globals.EXEC_MODE = globals.getExecMode(table[new_val])
            old_string = "[" + util.prettyDFMode(dictionary[value]) + "]"
            new_string =  "[" + util.prettyDFMode(table[new_val]) + "]"

            strings = self.correctStrings(old_string, new_string)

            self.oldSettings.append(strings[0] + " : " + label)
            self.changedSettings.append(strings[1] + " : " + label)

            dictionary[value] = table[new_val]
        else:
            globals.EXEC_MODE = globals.getExecMode(table[new_val])
            dictionary[value] = table[new_val]

