from Config import globals
from Config.settings import WEB_USER_PASS
from Util.cli import getSaveChoice
from Util.util import readFile, writeFile, fileExists, removeFile
import hashlib, re
from Component import GenericComponent

def isNetworkComponent(tag):
        return tag.find("DHCP") != None

class NetworkSettings(GenericComponent):
    def display(self, displayOnly = False):
        self.justDisplay = displayOnly

        network = globals.CONFIG_SETTINGS.find(isNetworkComponent)

        self.displayNetwork(network)
        timing = globals.CONFIG_SETTINGS.find('TIMING')
        self.displayTiming(timing)
        self.displaySecurity()

        return self.save(network, timing)


    def __init__(self, firstTime):
        self.maxAlignment = 36
        GenericComponent.__init__(self, firstTime)

    def displayNetwork(self, network):
        print "\n\n**** Network ****\n"
        DHCP = self.displayAndReadDHCP(network.DHCP)

        if DHCP is False:
            required = self.setRequired(network.IPEUSER['VALUE'])
            self.displayAndRead("IP Address", network.IPEUSER, 'VALUE', [1,2,3,4,5,6], self.cidrMatch, self.justDisplay, lengthLimit= 20, required=required)
            # Network and Broadcast are not user-defined
            self.displayAndRead("Gateway IP",  network.GATEWAY, 'VALUE', [1,2,3,4,5,6], self.ipMatch, self.justDisplay)


        self.displayAndRead("DNS Server Name #1", network.DNS, 'S1', [1,2,3,4,5,6], self.ipMatch, self.justDisplay)
        self.displayAndRead("DNS Server Name #2", network.DNS, 'S2', [1,2,3,4,5,6], self.ipMatch, self.justDisplay)
        self.displayAndRead("DNS Server Name #3", network.DNS, 'S3', [1,2,3,4,5,6], self.ipMatch, self.justDisplay)


    def displayTiming(self, timing):
        print "\n\n**** Timing ****\n"
        self.displayAndRead("NTP Server Name #1", timing.NTP, 'S1', [1,2,3,4,5,6] , "", self.justDisplay)
        self.displayAndRead("NTP Server Name #2", timing.NTP, 'S2', [1,2,3,4,5,6] , "", self.justDisplay)
        self.displayAndRead("NTP Server Name #3", timing.NTP, 'S3', [1,2,3,4,5,6] , "", self.justDisplay)
        self.displayAndRead("NTP Server Name #4", timing.NTP, 'S4', [1,2,3,4,5,6] , "", self.justDisplay)
        self.displayAndRead("NTP Server Name #5", timing.NTP, 'S5', [1,2,3,4,5,6] , "", self.justDisplay)


    def displayAndReadDHCP(self, dhcp):

        label = "IP (Dynamic|Static)"

        if self.justDisplay is True:
            self.displayOnly(label, dhcp['ONOFF'])
            return

        table = {
            "dynamic": "ON",
            "static": "OFF",
            "ON": "Dynamic",
            "OFF": "Static",
            "": ""
        }

        retTable = {
            "dynamic": True,
            "ON": True,
            "static": False,
            "OFF": False,
            "":""
        }

        display_string = label + " [" + table[dhcp['ONOFF']] + "] : "
        if len(display_string) > self.maxAlignment:
            spaces = 0
        else:
            diff = self.maxAlignment - len(display_string)
            spaces = diff

        with_space = " " * spaces + display_string
        new_val = raw_input(with_space).lower().replace(" ", "")

        new_val = self.validateAndRead("^(static|dynamic|Dynamic|Static)$", new_val, with_space).lower()



        ret = retTable[new_val]

        if not new_val:
            ret = retTable[dhcp['ONOFF']]

        if new_val == '':
            return ret
        elif self.firstTime is not True:
            old_string = "[" + table[dhcp['ONOFF']]  + "]"
            new_string = "[" + table[table[new_val]] + "]"

            strings = self.correctStrings(old_string, new_string)

            self.oldSettings.append(strings[0] + " : " + label)
            self.changedSettings.append(strings[1] + " : " + label)
            dhcp['ONOFF'] = table[new_val]
            return ret
        else:
            dhcp['ONOFF'] = table[new_val]
            return ret


    def displaySecurity(self):
        print "\n**** Security ****\n"
        self.displayOnly("Web User Name", self.readWebUser())
        self.displayAndReadWebPass()

    def displayAndReadWebPass(self):

        label = "Web Password"

        if self.justDisplay is True:
            self.displayOnly(label, "****")
            return

        four_stars = "****"


        if not fileExists("/tmp/password-set") and self.firstTime:
            four_stars = ""

        display_string = label + " [" + four_stars + "] : "
        if len(display_string) > self.maxAlignment:
            spaces = 0
        else:
            diff = self.maxAlignment - len(display_string)
            spaces = diff

        with_space = " " * spaces + display_string
        new_val = raw_input(with_space).replace(" ", "")

        new_val = self.validateWebPass(new_val, with_space)

        if self.firstTime is True:
            #write a temp file to let use know
            writeFile("/tmp/password-set", "set")

        if new_val == '' and self.firstTime is not True:
            #do nothing
            return
        else:
            self.changedSettings.append("The password for the web user has been changed")
            self.writeWebPass(new_val)


    def validateWebPass(self, value, with_space):
        """
        Validate the web password
        - it cannot be null when in wizardMode
        """
        while True:
            if self.firstTime is True and not value and not fileExists("/tmp/password-set"):
                print "Invalid entry, please re-enter"
                value = raw_input(with_space).replace(" ", "")

            else:
                return value



    def writeWebPass(self, password):
        """
        Write out the new password to the file.
        Create an SHA1 of the MD5 of the password
        in the same way that the PHP code does.
        """
        lines = readFile(WEB_USER_PASS)
        lines = lines[0].split("|;_;|")
        password = re.escape(password)
        m = hashlib.md5()
        m.update(password)
        md5 = m.hexdigest()
        m = hashlib.sha1()
        m.update(md5)
        new_pass = m.hexdigest()
        lines[1] = new_pass
        joined = '|;_;|'.join(lines)
        writeFile(WEB_USER_PASS, joined)

    def readWebUser(self):
        lines = readFile(WEB_USER_PASS)
        lines = lines[0].split("|;_;|")
        return lines[0]


    def save(self, network, timing):
        # If the use is in first time mode
        # just save the changes and continue
        if self.firstTime is False:

            changesMade = self.displayChanges()

            if not changesMade:
                choice = 'n'
                return choice


            choice = getSaveChoice()


            if choice == 's':
                globals.CONFIG_SETTINGS.find(isNetworkComponent).replaceWith(network)
                globals.CONFIG_SETTINGS.find('TIMING').replaceWith(timing)

            return choice
        else:
            globals.CONFIG_SETTINGS.find(isNetworkComponent).replaceWith(network)
            globals.CONFIG_SETTINGS.find('TIMING').replaceWith(timing)
