from Config import globals
from Util.cli import getSaveChoice
from Component import GenericComponent


class PremiumSettings(GenericComponent):
    def display(self, displayOnly = False):

        self.justDisplay = displayOnly
        darien = globals.CONFIG_SETTINGS.find('DARIEN')
        self.displayDarien(darien)

        # caps = globals.CONFIG_SETTINGS.find("CAPS")
        # self.displayCaps(caps)
        ##TODO: add possibility to upload license file!

        vtun = globals.CONFIG_SETTINGS.find('VTUN')
        self.displayTunneling(vtun)

        return self.save(darien, vtun)

    def __init__(self, firstTime):
        self.maxAlignment = 37
        self.nameMatch = "^[a-zA-Z0-9]+$"
        GenericComponent.__init__(self, firstTime)

    def displayDarien(self, darien):
        if(self.determineDisplay([2])):
            print "\n\n**** Darien Client ****\n"

            required = self.setRequired(darien.SERVERIP['VALUE'])

            self.displayAndRead("OSOP Zero Server IP", darien.SERVERIP, 'VALUE', [2], None, self.justDisplay, 100, required)

            required = self.setRequired(darien.PORT['VALUE'])

            self.displayAndRead("OSOP Zero Server Port Number", darien.PORT, 'VALUE', [2], self.numberMatch, self.justDisplay, 100, required)

    def displayTunneling(self, vtun):
        if(self.determineDisplay([1,2,3,4,5,6])):
            print "\n\n*** Tunneling ****\n"
            VTUN = self.displayAndReadTunneling(vtun.ONOFF)
            if VTUN is True:

                required = self.setRequired(vtun.SERVERIP['VALUE'])
                self.displayAndRead("Tunneling Server IP", vtun.SERVERIP, 'VALUE', [1,2,3,4,5,6], None, self.justDisplay, 100, required)

                required = self.setRequired(vtun.CLNTIP['VALUE'])
                self.displayAndRead("Tunneling Client IP", vtun.CLNTIP, 'VALUE', [1,2,3,4,5,6], self.ipMatch, self.justDisplay, 100, required)

                required = self.setRequired(vtun.CLNTPORT['VALUE'])
                self.displayAndRead("Tunneling Port", vtun.CLNTPORT, 'VALUE', [1,2,3,4,5,6], self.numberMatch, self.justDisplay, 100, required)

                required = self.setRequired(vtun.NAME['VALUE'])
                self.displayAndRead("Tunneling Name", vtun.NAME, 'VALUE', [1,2,3,4,5,6], self.nameMatch, self.justDisplay, 100, required)

                required = self.setRequired(vtun.PWORD['VALUE'])
                self.displayAndRead("Tunneling Password", vtun.PWORD, 'VALUE', [1,2,3,4,5,6], "", self.justDisplay, 100, required)



    def displayAndReadTunneling(self, vtun):
        label = "Tunneling (ON|OFF)"

        if self.justDisplay is True:
            self.displayOnly(label, vtun['VALUE'])
            return


        retTable = {
            "ON":True,
            "OFF":False,
            "":False
        }

        display_string = label + " [" + vtun['VALUE'] + "] : "

        if len(display_string) > self.maxAlignment:
            spaces = 0
        else:
            diff = self.maxAlignment - len(display_string)
            spaces = diff

        with_space = " " * spaces + display_string
        new_val = raw_input(with_space).upper().replace(" ", "")


        ret = retTable[new_val]

        if new_val == '':
            return retTable[vtun['VALUE']]
        elif self.firstTime is not True:
            old_string = "[" + vtun['VALUE'] + "]"
            new_string =  "[" + new_val + "]"

            strings = self.correctStrings(old_string, new_string)

            self.oldSettings.append(strings[0] + " : " + label)
            self.changedSettings.append(strings[1] + " : " + label)

            vtun['VALUE'] = new_val
            return ret
        else:
            vtun['VALUE'] = new_val
            return ret


    def save(self, darien, vtun):
        # If the use is in first time mode
        # just save the changes and continue
        if self.firstTime is False:
            changesMade = self.displayChanges()

            if not changesMade:
                choice = 'n'
                return choice

            choice = getSaveChoice()



            if choice == 's':
                globals.CONFIG_SETTINGS.find('DARIEN').replaceWith(darien)
                globals.CONFIG_SETTINGS.find('VTUN').replaceWith(vtun)
            return choice
        else:
            globals.CONFIG_SETTINGS.find('DARIEN').replaceWith(darien)
            globals.CONFIG_SETTINGS.find('VTUN').replaceWith(vtun)
