from Config import globals
from Component import GenericComponent
from Config.settings import MAC_ADDR_PROG, IP_ADDR_PROG, CHECK_SEEDLINK_EXEC, CHECK_CAPS_EXEC
from Util import util
import subprocess
import os

class SixaolaSettings(GenericComponent):
    def display(self):
        print "\nSixaola - Current Status"
        glob = globals.CONFIG_SETTINGS.find('GLOBAL')
        status = globals.STATUS.find('STATUS')
        trigger = globals.CONFIG_SETTINGS.find('TRIGGER')
        # used to make the output pretty for users
        pretty_table = {
            "SEEDLINK": "SeedLink Streamer",
            "DARIEN": "Darien Client",
            "DATA_LOGGER": "Data Logger",
            "EVENT_LOGGER": "Event Logger",
            "SEEDLINK_LOCAL_BACKUP": "SeedLink Streamer (With Local Backup)",
            "": ""
        }


        self.determineChannelsOrMac(glob)
        ips = self.getIPAddress().split()
        self.displayOnly("IP Address", ips[0])
        if len(ips) > 1:
            self.displayOnly("IP Discovery", ips[1])
        self.displayOnly("Data-Flow Mode", self.checkSeedlink(util.prettyDFMode(glob.EXECUTIONMODE['VALUE']), trigger.ONOFF['VALUE']))
        self.displayOnly("GPS", self.translateGPS(status))
        self.displayOnly("Time Since Last Restart", self.getUptime())
        self.displayOnly("Software Version", self.getSWHWVersion(status.SWVER['VALUE']))
        self.displayOnly("Firmware Version", self.getSWHWVersion(status.FWVER['VALUE']))
        utcTime = os.popen("date -u").read()
        self.displayOnly("System time", utcTime)
                

    def defineStationNameString(self, glob):
        name = glob.NTW['NAME'] + '.' + glob.STN['NAME']

        return name

    def checkSeedlink(self, execMode, trigger):
        if execMode == "Darien Client" and trigger == "OFF":
            return execMode

        if execMode == "CAPS":
            seed = subprocess.check_output(CHECK_CAPS_EXEC)
            return execMode + ", " + seed
            
        seed = subprocess.check_output(CHECK_SEEDLINK_EXEC).rstrip()

        return execMode + ", " + seed


    def parseTime(self, val):
        """
        If uptime gives an abbreviated time, extract the hours 
        and minutes from it.
        """
        val = val.split(":")
        hours = int(val[0])
        minutes = int(val[1])

        if hours > 0:
            string = str(hours) + " hours " + str(minutes) + " minutes"
        else:
            string = str(minutes) + " minutes"

        return string


    def getUptime(self):
        """
        Parse the result of the uptime command.
        There are a variety of possible outputs.
        """
        raw = subprocess.check_output('uptime')
        possible = ["days", "day", "min", "hrs"]
        if any(x in raw for x in possible):
            new = raw[0:raw.find("user")-5]
            new = new[new.find("up")+3:]
            if ":" in new:
                new = new.split(",")
                return new[0] +  " " + self.parseTime(new[1])

            return new # just return the empty string


        
        new = raw.split(",")[0]
        new = new[new.find("up")+2:].replace(" ", "")
        return self.parseTime(new)


    def translateGPS(self, status):
        """
        Check for the existence of the file in the GPS
        and see if it is on or off. No file means it is not locked
        """
        if not util.fileExists(status.GPSLOCK['VALUE']):
            return "Not Locked"

        lines = util.readFile(status.GPSLOCK['VALUE'])

        if not lines:
            return "Not Locked"

        if lines[0] == "ON":
            return "Locked"
        else:
            return "Not Locked"

    def determineChannelsOrMac(self, glob):
        if "DARIEN" in glob.EXECUTIONMODE['VALUE']:
            self.displayOnly("MAC Address", str(self.getMacAddress()))
        else:
            self.displayOnly("Station Name", self.defineStationNameString(glob))
            self.displayOnly("Channel Groups", self.translateChannelNames(glob))


    def getSWHWVersion(self, filename):
        """
        Read the filename and read the software/hardware version
        from the file.
        If the file doesn't exist - return empty string
        """
        if not util.fileExists(filename):
            return "1.0"
        
        lines = util.readFile(filename)

        if not lines:
            return "1.0"

        if not lines[0]:
            return "1.0"
        else:
            return lines[0]



    def getIPAddress(self):
        val = subprocess.Popen([IP_ADDR_PROG], stdout=subprocess.PIPE)
        output = val.stdout.read().rstrip()
        return output


    def getMacAddress(self):
        val = subprocess.Popen([MAC_ADDR_PROG], stdout=subprocess.PIPE)
        output = val.stdout.read()
        return output

    def translateChannelNames(self, glob):
        spsvTable = {
            "100": "EH",
            "50": "SH",
            "0" : ""
        }

        spsaTable = {
            "200": ", EN",
            "100": ", EN",
            "50": ", SN",
            "0": ""
        }

        return spsvTable[glob.SPSV['VALUE']] + spsaTable[glob.SPSA['VALUE']]

    def __init__(self, firstTime):
        self.maxAlignment = 26
        GenericComponent.__init__(self, firstTime)
