from Config import globals
from Components.Sixaola import SixaolaSettings
from Util.util import removeTmpWizard, removeFile
import sys

INITIAL = """
(A) Options:
   G) Overview Config Settings
   E) Edit Config Settings
   Q) Quit or Restart """

B_OPTIONS = """
(B) Options:
   I) Instrumentation
   N) Networking
   D) Data
   P) Premium
   Q) Quit or Restart """

E_OPTIONS = """
(E) Options:
   S) Save
   C) Cancel """

F_OPTIONS = """
(F) Options:
   R) Restart
   E) Edit More
   Q) Quit or Restart """

WIZARD_INITIAL = """First Time Configuration Wizard
   (A) Options:
      S) Start Wizard
      Q) Quit Wizard """

WIZARD_FINAL = """
   (B) Options:
      R) Save and Restart
      C) Change Settings """


SAVE_CHOICE = False

def inputError(): 
    val = raw_input("Unrecognized option. Please try again: ").lower()
    return val

def initialOptions(settingsDict, sixaola, printAndQuit = False):
    if printAndQuit:
        sixaola = SixaolaSettings(False)
        sixaola.display();
        settingsDict['g'].display()
        sys.exit()

    if sixaola is True:
        sixaola = SixaolaSettings(False)
        sixaola.display();

    print INITIAL
    choice = raw_input('> ').lower()
    quit = None
    while True:
        if quit == 'y':
            print "Exiting..."
            endState(False)
        elif quit == 'n':
            initialOptions(settingsDict)
        if choice == 'g':
            settingsDict[choice].display()
            initialOptions(settingsDict, True)
        elif choice == 'e':
            bChoices(settingsDict)
            choice = raw_input('> ').lower()
        elif choice == 'q':
            quitChoice()
        else:
            choice = inputError()



def bChoices(settingsDict):
    print B_OPTIONS
    component_choice = raw_input('> ').lower()

    while True:
        if component_choice == 'q':
            quitChoice()
        elif component_choice not in ['i', 'n', 'd', 'p']:
            component_choice = inputError()
        else:
            choice = settingsDict[component_choice].display()
            bChoicesLoop(settingsDict, component_choice, choice)

   
        
def bChoicesLoop(settingsDict, component_choice, choice):
    while True:
        if choice == 's':
            globals.write()
            global SAVE_CHOICE
            SAVE_CHOICE = True # used by quitChoice to display restart message
            fChoices(settingsDict)
        elif choice == 'c' or choice == 'n':
            bChoices(settingsDict)
        elif choice == 'q':
            quitChoice()
        elif not choice:
            choice = inputError()
        else:
            choice = inputError()    

def fChoices(settingsDict):
    print F_OPTIONS
    choice = raw_input('> ').lower()
    if choice == 'r':
        endState(True)
    elif choice == 'e':
        bChoices(settingsDict)
    elif choice == 'q':
        quitChoice()
    else:
        choice = inputError()



def restartQuitOptions():
    choice = raw_input("Saved changes only take effect at Sixaola restart. Restart now [y/N]? ").lower()
    while True:
        if choice == 'y':
            endState(True)
        elif choice == 'n' or not choice:
            endState(False)
        else:
            choice = inputError()

def quitChoice():
    if SAVE_CHOICE is True:
        restartQuitOptions()
    elif SAVE_CHOICE is False:
        endState(False)

    

def getSaveChoice():
    print E_OPTIONS
    choice = raw_input('> ').lower()
    return choice
   
def wizardMode(settingsDict):
    removeFile("/tmp/password-set")
    print WIZARD_INITIAL
    choice = raw_input('> ').lower()

    while True:
        if choice == 's':
            # get wizarding
            wizardTime(settingsDict)
        elif choice == 'q':
            print "Exiting..."
            endState(False)
        else:
            choice = inputError()

def wizardReview(settingsDict):
    """
    Ask the user if they want to look at their settings or not
    """
    choice = raw_input('\nReview settings [y/N]? ').lower()
    if choice == 'y':
        print "\n**** Selected Settings Review ****"
        for key in settingsDict:
            if key is not 'g':
                settingsDict[key].display(True) #if True we use displayOnly
    elif choice == 'n' or not choice:
        return
    else:
        choice = inputError()
        

def wizardTime(settingsDict):
    # go through the settings in wizard
    for key in settingsDict:
        if key is not 'g':
            settingsDict[key].display()
            globals.write()

    wizardReview(settingsDict)
    wizardEnd(settingsDict)

        
def wizardEnd(settingsDict):
    # print the settings
    # and ask to save or not

    print WIZARD_FINAL
    choice = raw_input('> ').lower()


    while True:
        if choice == 'r':
            removeTmpWizard()
            removeFile("/tmp/password-set")
            endState(True)
        elif choice == 'c':
            wizardTime(settingsDict)
        else:
            choice = inputError()
    

def endState(restart):
    if restart is True:
        globals.write()
        globals.shutdown()
        exit()
    else:
        globals.write()
        exit()


