#!/usr/bin/env python2

import os
import sys
from xseed import Parser
from xml.dom.minidom import parse, parseString

#####################################################################

def getSixaolaVersion():
    verFile = "/opt/settings/sc-xml/FW-ver.txt"
    version = open(verFile, 'r').readline()
    return version.split(".")[0]


def getGPSCoords():
    gpsPath = "/opt/settings/sc-xml/GPSCoords.txt"
    if os.path.exists(gpsPath):
        with open(gpsPath, 'r') as f:
            gps = f.readlines()
        coords = map(float, gps)
        return coords
    else:
        print 'WARNING: using artificial coordinates'
        return (0, 0, 0)


def read_attr(dom, name, attr):

    temp = dom.getElementsByTagName(name)
    if temp == []:
        print 'No name %s in DOM' % name
        return None

    return temp[0].getAttribute(attr).strip()


def read_params(cname):

    try:
        dom = parse(cname)
    except:
        print 'Cannot read %s' % cname
        return None

    net = read_attr(dom, 'NTW', 'NAME')
    sta = read_attr(dom, 'STN', 'NAME')
    loc = read_attr(dom, 'LOC', 'NAME')
    spa = read_attr(dom, 'SPSA', 'VALUE')
    spv = read_attr(dom, 'SPSV', 'VALUE')

    spa = int(spa)
    spv = int(spv)

    if spa == 100 or spa == 200:
        acc = 'EN'
    elif spa == 50:
        acc = 'SN'
    else:
        acc = 'not required'

    if spv == 100:
        vel = 'EH'
    elif spv == 50:
        vel = 'SH'
    else:
        vel = 'not required'


    [lat, lon, hei] = getGPSCoords()
    #lat =  8.78
    #lon = -82.63
    #hei =  1400

    return { 'NET': net, 'STA': sta, 'LOC': loc,
             'SPA': spa, 'SPV': spv, 'ACC': acc, 'VEL': vel,
             'LAT': lat, 'LON': lon, 'HEI': hei }

#####################################################################

def blockette(node):

    if node.nodeType == node.ELEMENT_NODE:
        if node.hasAttribute('blockette'):
            return node.getAttribute('blockette')

    return None


def get_child(node, name):

    if node is None or node.nodeType != node.ELEMENT_NODE:
        return None

    temp = node.getElementsByTagName(name)
    if len(temp) > 0:
        return temp[0]
    else:
        print 'No <%s> child in node <%s>' % (name, node.localName)
        return None


def get_value(node):

    if node is None:
        return None

    if node.nodeType == node.TEXT_NODE:
        return node.nodeValue

    for child in node.childNodes:
        if child.nodeType == child.TEXT_NODE:
            return child.nodeValue

    return None


def set_value(node, val):

    if node is None or node.nodeType != node.ELEMENT_NODE:
        return

    for child in node.childNodes:
        if child.nodeType == child.TEXT_NODE:
            child.nodeValue = val


def patch050(node, pars):

    temp = get_child(node, 'latitude')
    set_value(temp, pars['LAT'])

    temp = get_child(node, 'longitude')
    set_value(temp, pars['LON'])

    temp = get_child(node, 'elevation')
    set_value(temp, pars['HEI'])

    temp = get_child(node, 'network_code')
    set_value(temp, pars['NET'])

    temp = get_child(node, 'station_call_letters')
    set_value(temp, pars['STA'])


def check052(node, pars):

    spa = pars['SPA']
    spv = pars['SPV']
    acc = pars['ACC']
    vel = pars['VEL']

    temp = get_child(node, 'channel_identifier')
    chan = get_value(temp)

    temp = get_child(node, 'sample_rate')
    rate = get_value(temp)
    rate = float(rate)

    if acc in chan and rate == spa:
        return True

    if vel in chan and rate == spv:
        return True

    return False


def patch052(node, pars):

    temp = get_child(node, 'latitude')
    set_value(temp, pars['LAT'])

    temp = get_child(node, 'longitude')
    set_value(temp, pars['LON'])

    temp = get_child(node, 'elevation')
    set_value(temp, pars['HEI'])

    temp = get_child(node, 'location_identifier')
    set_value(temp, '')


def patch_xseed(xml, pars):

    try:
        dom = parseString(xml)
    except:
        return None

    root = dom.documentElement

    temp = get_child(root, 'generic_abbreviation')
    temp = get_child(temp, 'abbreviation_description')
    set_value(temp, pars['NET'])

    temp = get_child(root, 'station_control_header')
    if temp is None:
        return None

    active = False

    for node in temp.childNodes:

        blkt = blockette(node)

        if blkt is None:
            continue

        if blkt == '050':
            patch050(node, pars)
            continue

        if blkt == '052':
            active = check052(node, pars)
            if active:
                patch052(node, pars)

        if not active:
            node.parentNode.removeChild(node)

    return dom.toxml(encoding='UTF-8')

#####################################################################

def read_inventory(ifile):
    try:
        with open(ifile, 'r') as f:
            xml = f.read()
    except:
        return None

    try:
        dom = parseString(xml)
    except:
        return None

    return dom


def channel_list(pars):

    ret = []

    spa = int(pars['SPA'])
    if spa == 100 or spa == 200:
        ret.append('EN')
    elif spa == 50:
        ret.append('SN')
    else:
        pass

    spv = int(pars['SPV'])
    if spv == 100:
        ret.append('EH')
    elif spv == 50:
        ret.append('SH')
    else:
        pass

    return ret


def filter_inventory(dom, pars):

    chans = channel_list(pars) 

    spa = int(pars['SPA'])
    spv = int(pars['SPV'])

    # remove streams that do not match
    nodes = dom.getElementsByTagName('stream')
    for node in nodes:
        code = node.getAttribute('code')
        if code[-3:-1] not in chans:
            node.parentNode.removeChild(node)
            continue

        rate = node.getElementsByTagName('sampleRateNumerator')[0]
        rate = get_value(rate)
        rate = int(rate)

        if   code[1] == 'H':
            if rate != spv:
                #print node, code, rate, spv
                node.parentNode.removeChild(node)
        elif code[1] == 'N':
            if rate != spa:
                #print node, code, rate, spa
                node.parentNode.removeChild(node)
        else:
            pass

    # glue sensorLocations into one
    nodes = dom.getElementsByTagName('stream')
    slocs = dom.getElementsByTagName('sensorLocation')
    for node in nodes:
        node.parentNode.removeChild(node)

    for sloc in slocs[1:]:
        sloc.parentNode.removeChild(sloc)

    for node in nodes:
        slocs[0].appendChild(node)

    # delete unneeded sensors 
    streams = dom.getElementsByTagName('stream')

    tokeep = map(lambda x: x.getAttribute('sensor'), streams)
    nodes = dom.getElementsByTagName('sensor')
    for node in nodes:
        pubID = node.getAttribute('publicID')
        if pubID not in tokeep:
            node.parentNode.removeChild(node)

    # delete unneeded dataloggers
    tokeep = map(lambda x: x.getAttribute('datalogger'), streams)
    nodes = dom.getElementsByTagName('datalogger')
    for node in nodes:
        pubID = node.getAttribute('publicID')
        if pubID not in tokeep:
            node.parentNode.removeChild(node)

    # delete unneeded PAZs
    sensors = dom.getElementsByTagName('sensor')
    tokeep = map(lambda x: x.getAttribute('response'), sensors)
    nodes = dom.getElementsByTagName('responsePAZ')
    for node in nodes:
        pubID = node.getAttribute('publicID')
        if pubID not in tokeep:
            node.parentNode.removeChild(node)

    # delete unneeded FIRs
    loggers = dom.getElementsByTagName('datalogger')
    tokeep = []
    for logger in loggers:
        keep = logger.getElementsByTagName('digitalFilterChain')[0]
        keep = get_value(keep)
        tokeep.append(keep)

    nodes = dom.getElementsByTagName('responseFIR')
    for node in nodes:
        pubID = node.getAttribute('publicID')
        if pubID not in tokeep:
            node.parentNode.removeChild(node)

    # fix names
    nodes = dom.getElementsByTagName('sensor')
    for node in nodes:
        name = node.getAttribute('name')
        toks = name.split('.')
        toks[-1] = toks[-1][:2]
        name = '.'.join(toks)
        node.setAttribute('name', name)

    nodes = dom.getElementsByTagName('datalogger')
    for node in nodes:
        name = node.getAttribute('name')
        toks = name.split('.')
        toks[-1] = toks[-1][:3]
        name = '.'.join(toks)
        node.setAttribute('name', name)

    nodes = dom.getElementsByTagName('responsePAZ')
    for node in nodes:
        name = node.getAttribute('name')
        toks = name.split('.')
        toks[-1] = toks[-1][:2]
        name = '.'.join(toks)
        node.setAttribute('name', name)

def patch_inventory(dom, pars):

    nodes = dom.getElementsByTagName('latitude')
    for node in nodes:
        set_value(node, pars['LAT'])

    nodes = dom.getElementsByTagName('longitude')
    for node in nodes:
        set_value(node, pars['LON'])

    nodes = dom.getElementsByTagName('elevation')
    for node in nodes:
        set_value(node, pars['HEI'])

    nodes = dom.getElementsByTagName('network')
    for node in nodes:
        node.setAttribute('code', pars['NET'])

    nodes = dom.getElementsByTagName('station')
    for node in nodes:
        node.setAttribute('code', pars['STA'])

    nodes = dom.getElementsByTagName('sensor')
    for node in nodes:
        nodes1 = node.getElementsByTagName('description')
        for node1 in nodes1:
            set_value(node1, pars['NET'])

    nodes = dom.getElementsByTagName('model')
    for node in nodes:
        set_value(node, pars['NET'])

    nodes = dom.getElementsByTagName('affiliation')
    for node in nodes:
        set_value(node, pars['NET'])

    nodes = dom.getElementsByTagName('sensor')
    for node in nodes:
        name = node.getAttribute('name')
        toks = name.split('.')
        toks[0] = pars['STA']
        name = '.'.join(toks)
        node.setAttribute('name', name)

    nodes = dom.getElementsByTagName('datalogger')
    for node in nodes:
        name = node.getAttribute('name')
        toks = name.split('.')
        toks[0] = pars['STA']
        name = '.'.join(toks)
        node.setAttribute('name', name)

    nodes = dom.getElementsByTagName('datalogger')
    for node in nodes:
        name = node.getAttribute('name')
        nodes1 = node.getElementsByTagName('description')
        for node1 in nodes1:
            set_value(node1, name)

    nodes = dom.getElementsByTagName('network')
    for node in nodes:
        nodes1 = node.getElementsByTagName('description')
        for node1 in nodes1:
            set_value(node1, pars['NET'])


    nodes = dom.getElementsByTagName('responsePAZ')
    for node in nodes:
        name = node.getAttribute('name')
        toks = name.split('.')
        toks[0] = pars['STA']
        name = '.'.join(toks)
        node.setAttribute('name', name)

    nodes = dom.getElementsByTagName('responseFIR')
    for node in nodes:
        name = node.getAttribute('name')
        toks = name.split('.')
        toks[0] = pars['STA']
        name = '.'.join(toks)
        node.setAttribute('name', name)

    filter_inventory(dom, pars)


def write_inventory(ofile, dom):
    xml = dom.toxml(encoding='UTF-8')
    with open(ofile, 'w') as f:
        f.write(xml)

#####################################################################

print ''
print 'BACKEND: fill in Sixaola information into response files and'
print 'provide output as RESP, dataless SEED, XSEED, or FDSN Station XML'
print ''
print 'Usage: %s <command> <input file>' % sys.argv[0]
print ''
print 'where'
print ''
print '<command>      is one of:'
print '  -r to provide RESP'
print '  -s to provide SEED'
print '  -x to provide XSEED'
print '  -l to provide Seiscomp Inventory XML'
print '<input file>   is either a dataless SEED or XSEED file'
print ''

args = sys.argv

if len(args) != 3:
    print 'Should provide 2 arguments'
    exit(1)

COMMD = args[1]
INAME = args[2]
#ONAME = args[3]
CNAME = '/opt/settings/sc-xml/SC-Configuration.xml'

if COMMD not in ('-r', '-s', '-x', '-l'):
    print 'Invalid <command>: %s' % COMMD
    exit(1)

if not os.path.isfile(INAME):
    print 'No <input file> %s' % INAME
    exit(1)

if not os.path.isfile(CNAME):
    print 'No config file: %s' % CNAME
    exit(1)


print 'Using command:     %s' % COMMD
print 'Using input file:  %s' % INAME



print '--- Reading Sixaola information'

pars = read_params(CNAME)
if pars is None:
    print 'Cannot read %s' % CNAME
    exit(1)

ver = getSixaolaVersion()

baseName= "%s.%s.sixaola%s" % (pars["NET"], pars["STA"], ver)

if   COMMD == '-r':
    ONAME = baseName
    print 'Writing RESP to dir: %s' % ONAME
elif COMMD == '-x':
    ONAME = baseName + '.xseed'
    print 'Writing XSEED file %s' % ONAME
elif COMMD == '-s':
    ONAME = baseName + '.seed'
    print 'Writing SEED file %s' % ONAME
elif COMMD == '-l':
    ONAME = baseName + '.xml'
    print 'Writing Inventory XML file %s' % ONAME
else:
    exit(1)

print 'Using config file: %s' % CNAME
print ''

print '--- Reading input file'

parser = None

try:
    parser = Parser(INAME)
except:
    print 'Cannot read %s' % INAME
    exit(1)

xml = parser.getXSEED()

del parser


print '--- Filling in Sixaola information'

xml = patch_xseed(xml, pars)
if xml is None:
    print 'Cannot patch xml'
    exit(1)

try:
    parser = Parser(xml)
except:
    print 'Cannot read xml'
    exit(1)


if COMMD == '-r':
    print '--- Writing RESP to output directory'
    if os.path.isfile(ONAME):
        print 'Found file %s. Please remove and re-run.' % ONAME
        exit(1)

    if not os.path.isdir(ONAME):
        os.makedirs(ONAME)

    parser.writeRESP(ONAME)

if COMMD == '-s':
    print '--- Writing dataless SEED to output file'
    parser.writeSEED(ONAME)

if COMMD == '-x':
    print '--- Writing XSEED to output file'
    parser.writeXSEED(ONAME)

if COMMD == '-l':
    print '--- Converting and writing Inventory XML file'

    ifile = INAME.replace('dataless', 'xml')
    if not os.path.isfile(ifile):
        print 'Cannot read %s. Exit' % ifile
    dom = read_inventory(ifile)
    patch_inventory(dom, pars)
    write_inventory(ONAME, dom)



if (pars["LAT"], pars["LON"]) == (0, 0):
    with open("README.txt", 'w') as readme:
        readme.write("No GPS coordinates available.\n Please edit files manually and add proper coordinates.")

print '--- Done. See %s for output.' % ONAME
print ''
